//=====================================================
// File   :  ublas_interface.hh
// Author :  L. Plagne <laurent.plagne@edf.fr)>
// Copyright (C) EDF R&D,  lun sep 30 14:23:27 CEST 2002
//=====================================================
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
#ifndef FLENS_INTERFACE_HH
#define FLENS_INTERFACE_HH

#include <flens/flens.h>

using namespace flens;

template <typename  real>
class FLENS_interface
{
    public :

        typedef real real_type ;

        typedef std::vector<real>  stl_vector;
        typedef std::vector<stl_vector > stl_matrix;

        typedef GeMatrix<FullStorage<double, ColMajor> >  gene_matrix;
        typedef DenseVector<Array<double> >               gene_vector;

        static inline
        std::string
        name( void )
        {
            return "FLENS-ATLAS";
        }

        static void
        free_matrix(gene_matrix &A, int N)
        {
            return;
        }

        static void
        free_vector(gene_vector &B)
        {
            return;
        }

        static inline void
        matrix_from_stl(gene_matrix &A, stl_matrix &A_stl)
        {
            A.resize(A_stl.size(),A_stl[0].size());

            for (int i=0, I=1; I<=A.numRows(); ++i, ++I) {
                for (int j=0, J=1; J<=A.numCols(); ++j, ++J) {
                    A(I,J) = A_stl[i][j];
                }
            }
        }

        static inline void
        vector_from_stl(gene_vector &B, stl_vector &B_stl)
        {
            B.resize(B_stl.size());

            for (int i=0, I=1; I<=B.length(); ++i, ++I){
                B(I)=B_stl[i];
            }
        }

        static inline void
        vector_to_stl(gene_vector &B, stl_vector &B_stl)
        {
            for (int i=0, I=1; I<=B.length(); ++i, ++I) {
                B_stl[i] = B(I);
            }
        }

        static inline void
        matrix_to_stl(gene_matrix &A, stl_matrix &A_stl)
        {
            int N=A_stl.size();

            for (int i=0, I=1; I<=N; ++i, ++I) {
                A_stl[i].resize(N);
                for (int j=0, J=1; J<=N; ++j, ++J) {
                    A_stl[i][j] = A(I,J);
                }
            }
        }

        static inline void
        copy_vector(const gene_vector &source, gene_vector &cible, int N)
        {
            for (int i=1; i<=N; ++i) {
                cible(i)=source(i);
            }
        }

        static inline void
        copy_matrix(const gene_matrix &source, gene_matrix &cible, int N)
        {
            for (int i=1; i<=N; ++i) {
                for (int j=1; j<=N; ++j) {
                    cible(i,j) = source(i,j);
                }
            }
        }

        static inline void
        matrix_vector_product(gene_matrix &A, gene_vector &b,
                              gene_vector &x, int N)
        {
            x = A*b;
        }

        static inline void
        matrix_matrix_product(gene_matrix &A, gene_matrix &B,
                              gene_matrix &X, int N)
        {
            X = A*B;
        }

        static inline void
        axpy(const real coef, const gene_vector &x, gene_vector &y, int N)
        {
            y+=coef*x;
        }

        static inline void
        ata_product(gene_matrix &A, gene_matrix &X, int N)
        {
            X = transpose(A)*A;
        }

        static inline void
        aat_product(gene_matrix &A, gene_matrix &X, int N)
        {
            X = A*transpose(A);
        }

};

#endif  // FLENS_INTERFACE_HH
